@@ -3,7 +3,7 @@
 /*
     Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2023 Particular Programs Ltd.
+    Copyright 2007-2024 Particular Programs Ltd.
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -215,7 +215,7 @@ public:
 class Condition
 {
 public:
-    Condition(std::string name) { }
+    Condition(std::string) { }
     ~Condition() { }
     
     void lock() { }
