@@ -3,7 +3,7 @@
 /*
     Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2022 Particular Programs Ltd.
+    Copyright 2007-2024 Particular Programs Ltd.
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -223,6 +223,8 @@ R2Stretcher::reset()
     m_mode = JustCreated;
     if (m_phaseResetAudioCurve) m_phaseResetAudioCurve->reset();
     if (m_silentAudioCurve) m_silentAudioCurve->reset();
+    m_expectedInputDuration = 0;
+    m_maxProcessSize = 0;
     m_inputDuration = 0;
     m_silentHistory = 0;
 
@@ -305,12 +307,21 @@ R2Stretcher::setExpectedInputDuration(size_t samples)
 void
 R2Stretcher::setMaxProcessSize(size_t samples)
 {
+    m_log.log(2, "R2Stretcher::setMaxProcessSize", samples);
     if (samples <= m_maxProcessSize) return;
+
+    m_log.log(2, "R2Stretcher::setMaxProcessSize: increasing from, to", m_maxProcessSize, samples);
     m_maxProcessSize = samples;
 
     reconfigure();
 }
 
+size_t
+R2Stretcher::getProcessSizeLimit() const
+{
+    return 524288;
+}
+
 void
 R2Stretcher::setKeyFrameMap(const std::map<size_t, size_t> &mapping)
 {
@@ -667,6 +678,7 @@ R2Stretcher::configure()
             size_t rbs = 
                 lrintf(ceil((m_increment * m_timeRatio * 2) / m_pitchScale));
             if (rbs < m_increment * 16) rbs = m_increment * 16;
+            if (rbs < m_aWindowSize * 2) rbs = m_aWindowSize * 2;
             m_channelData[c]->setResampleBufSize(rbs);
         }
     }
@@ -823,14 +835,28 @@ size_t
 R2Stretcher::getPreferredStartPad() const
 {
     if (!m_realtime) return 0;
-    return m_aWindowSize/2;
+
+    size_t pad = m_aWindowSize / 2;
+    
+    if (resampleBeforeStretching()) {
+        return size_t(ceil(pad * m_pitchScale));
+    } else {
+        return pad;
+    }
 }
 
 size_t
 R2Stretcher::getStartDelay() const
 {
     if (!m_realtime) return 0;
-    return lrint((m_aWindowSize/2) / m_pitchScale);
+
+    size_t pad = m_aWindowSize / 2;
+
+    if (resampleBeforeStretching()) {
+        return pad;
+    } else {
+        return size_t(ceil(pad / m_pitchScale));
+    }
 }
 
 void
@@ -967,6 +993,8 @@ R2Stretcher::study(const float *const *input, size_t samples, bool final)
         mixdown = input[0];
     }
 
+    std::vector<float> optionalFoldBuffer;
+    
     while (consumed < samples) {
 
 	size_t writable = inbuf.getWriteSpace();
@@ -1016,15 +1044,18 @@ R2Stretcher::study(const float *const *input, size_t samples, bool final)
 
                 // Note that we can't do this in-place.  Pity
 
-                float *tmp = (float *)alloca
-                    (std::max(m_fftSize, m_aWindowSize) * sizeof(float));
+                // This doesn't have to be allocation-free, as we're
+                // in offline-mode study
+
+                optionalFoldBuffer.resize(std::max(m_fftSize, m_aWindowSize));
 
                 if (m_aWindowSize > m_fftSize) {
                     m_afilter->cut(cd.accumulator);
                 }
 
-                cutShiftAndFold(tmp, m_fftSize, cd.accumulator, m_awindow);
-                v_copy(cd.accumulator, tmp, m_fftSize);
+                cutShiftAndFold(optionalFoldBuffer.data(), m_fftSize,
+                                cd.accumulator, m_awindow);
+                v_copy(cd.accumulator, optionalFoldBuffer.data(), m_fftSize);
             }
 
             m_studyFFT->forwardMagnitude(cd.accumulator, cd.fltbuf);
@@ -1032,8 +1063,6 @@ R2Stretcher::study(const float *const *input, size_t samples, bool final)
             float df = m_phaseResetAudioCurve->processFloat(cd.fltbuf, m_increment);
             m_phaseResetDf.push_back(df);
 
-//            cout << m_phaseResetDf.size() << " [" << final << "] -> " << df << " \t: ";
-
             df = m_silentAudioCurve->processFloat(cd.fltbuf, m_increment);
             bool silent = (df > 0.f);
             if (silent) {
@@ -1041,8 +1070,6 @@ R2Stretcher::study(const float *const *input, size_t samples, bool final)
             }
             m_silence.push_back(silent);
 
-//            cout << df << endl;
-
             // We have augmented the input by m_aWindowSize/2 so that
             // the first chunk is centred on the first audio sample.
             // We want to ensure that m_inputDuration contains the
@@ -1054,7 +1081,7 @@ R2Stretcher::study(const float *const *input, size_t samples, bool final)
             inbuf.skip(m_increment);
 	}
     }
-
+    
     if (final) {
         int rs = inbuf.getReadSpace();
         m_inputDuration += rs;
@@ -1202,6 +1229,10 @@ R2Stretcher::getSamplesRequired() const
             }
         }
     }
+
+    if (resampleBeforeStretching() && m_pitchScale > 1.0) {
+        reqd = size_t(ceil(double(reqd) * m_pitchScale));
+    }
     
     return reqd;
 }    
@@ -1211,6 +1242,8 @@ R2Stretcher::process(const float *const *input, size_t samples, bool final)
 {
     Profiler profiler("R2Stretcher::process");
 
+    m_log.log(3, "process entering, samples and final", samples, final);
+
     if (m_mode == Finished) {
         m_log.log(0, "R2Stretcher::process: Cannot process again after final chunk");
         return;
@@ -1273,10 +1306,16 @@ R2Stretcher::process(const float *const *input, size_t samples, bool final)
                                           consumed[c],
                                           samples - consumed[c],
                                           final);
+            if (c == 0) {
+                m_log.log(3, "consumed channel 0, consumed and samples now", consumed[c], samples);
+            }
             if (consumed[c] < samples) {
                 allConsumed = false;
             } else {
                 if (final) {
+                    if (c == 0) {
+                        m_log.log(2, "final is true, setting input size", m_channelData[c]->inCount);
+                    }
                     m_channelData[c]->inputSize = m_channelData[c]->inCount;
                 }
             }
@@ -1311,10 +1350,10 @@ R2Stretcher::process(const float *const *input, size_t samples, bool final)
         }
 #endif
 
-        m_log.log(2, "process looping");
+        m_log.log(3, "process looping");
     }
 
-    m_log.log(2, "process returning");
+    m_log.log(3, "process returning");
 
     if (final) m_mode = Finished;
 }
